@@ -74,37 +74,37 @@
 #include <freetds/configs.h>
 #include <freetds/utils/string.h>
 #include <freetds/utils.h>
-#include "replacements.h"
+#include <freetds/replacements.h>
 
-static int tds_config_login(TDSLOGIN * connection, TDSLOGIN * login);
-static int tds_config_env_tdsdump(TDSLOGIN * login);
+static bool tds_config_login(TDSLOGIN * connection, TDSLOGIN * login);
+static bool tds_config_env_tdsdump(TDSLOGIN * login);
 static void tds_config_env_tdsver(TDSLOGIN * login);
 static void tds_config_env_tdsport(TDSLOGIN * login);
-static int tds_config_env_tdshost(TDSLOGIN * login);
+static bool tds_config_env_tdshost(TDSLOGIN * login);
 static bool tds_read_conf_sections(FILE * in, const char *server, TDSLOGIN * login);
-static int tds_read_interfaces(const char *server, TDSLOGIN * login);
-static int parse_server_name_for_port(TDSLOGIN * connection, TDSLOGIN * login, bool update_server);
+static bool tds_read_interfaces(const char *server, TDSLOGIN * login);
+static bool parse_server_name_for_port(TDSLOGIN * connection, TDSLOGIN * login, bool update_server);
 static int tds_lookup_port(const char *portname);
-static void tds_config_encryption(const char * value, TDSLOGIN * login);
+static bool tds_config_encryption(const char * value, TDSLOGIN * login);
 
-static char *interf_file = NULL;
+static tds_dir_char *interf_file = NULL;
 
 #define TDS_ISSPACE(c) isspace((unsigned char ) (c))
 
+const char STD_DATETIME_FMT[] = "%b %e %Y %I:%M%p";
+
 #if !defined(_WIN32) && !defined(DOS32X)
-       const char STD_DATETIME_FMT[] = "%b %e %Y %I:%M%p";
 static const char pid_config_logpath[] = "/tmp/tdsconfig.log.%d";
-static const char freetds_conf[] = "%s/etc/freetds.conf";
+static const char freetds_conf[] = "etc/freetds.conf";
 static const char location[] = "(from $FREETDS/etc)";
 static const char pid_logpath[] = "/tmp/freetds.log.%d";
 static const char interfaces_path[] = "/etc/freetds";
 #else
-       const char STD_DATETIME_FMT[] = "%b %d %Y %I:%M%p"; /* msvcr80.dll does not support %e */
-static const char pid_config_logpath[] = "c:\\tdsconfig.log.%d";
-static const char freetds_conf[] = "%s\\freetds.conf";
+static const tds_dir_char pid_config_logpath[] = L"c:\\tdsconfig.log.%d";
+static const tds_dir_char freetds_conf[] = L"freetds.conf";
 static const char location[] = "(from $FREETDS)";
-static const char pid_logpath[] = "c:\\freetds.log.%d";
-static const char interfaces_path[] = "c:\\";
+static const tds_dir_char pid_logpath[] = L"c:\\freetds.log.%d";
+static const tds_dir_char interfaces_path[] = L"c:\\";
 #endif
 
 /**
@@ -138,32 +138,26 @@ TDSLOGIN *
 tds_read_config_info(TDSSOCKET * tds, TDSLOGIN * login, TDSLOCALE * locale)
 {
 	TDSLOGIN *connection;
-	char *s;
-	char *path;
-	pid_t pid;
+	tds_dir_char *s;
 	int opened = 0;
 	bool found;
-	struct addrinfo *addrs;
 
 	/* allocate a new structure with hard coded and build-time defaults */
-	connection = tds_alloc_login(0);
+	connection = tds_alloc_login(false);
 	if (!connection || !tds_init_login(connection, locale)) {
 		tds_free_login(connection);
 		return NULL;
 	}
 
-	s = getenv("TDSDUMPCONFIG");
+	s = tds_dir_getenv(TDS_DIR("TDSDUMPCONFIG"));
 	if (s) {
 		if (*s) {
-			opened = tdsdump_open(s);
+			opened = tdsdump_topen(s);
 		} else {
-			pid = getpid();
-			if (asprintf(&path, pid_config_logpath, (int) pid) >= 0) {
-				if (*path) {
-					opened = tdsdump_open(path);
-				}
-				free(path);
-			}
+			tds_dir_char path[TDS_VECTOR_SIZE(pid_config_logpath) + 22];
+			pid_t pid = getpid();
+			tds_dir_snprintf(path, TDS_VECTOR_SIZE(path), pid_config_logpath, (int) pid);
+			opened = tdsdump_topen(path);
 		}
 	}
 
@@ -213,6 +207,7 @@ tds_read_config_info(TDSSOCKET * tds, TDSLOGIN * login, TDSLOCALE * locale)
 	}
 	
 	if (opened) {
+		struct addrinfo *addrs;
 		char tmp[128];
 
 		tdsdump_log(TDS_DBG_INFO1, "Final connection parameters:\n");
@@ -248,10 +243,9 @@ tds_read_config_info(TDSSOCKET * tds, TDSLOGIN * login, TDSLOCALE * locale)
 		/* tdsdump_log(TDS_DBG_INFO1, "\t%20s = %s\n", "capabilities", tds_dstr_cstr(&connection->capabilities)); 
 			(not null terminated) */
 		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %s\n", "database", tds_dstr_cstr(&connection->database));
-		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %s\n", "dump_file", tds_dstr_cstr(&connection->dump_file));
+		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %" tdsPRIdir "\n", "dump_file", connection->dump_file);
 		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %x\n", "debug_flags", connection->debug_flags);
 		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %d\n", "text_size", connection->text_size);
-		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %d\n", "emul_little_endian", connection->emul_little_endian);
 		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %s\n", "server_realm_name", tds_dstr_cstr(&connection->server_realm_name));
 		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %s\n", "server_spn", tds_dstr_cstr(&connection->server_spn));
 		tdsdump_log(TDS_DBG_INFO1, "\t%20s = %s\n", "cafile", tds_dstr_cstr(&connection->cafile));
@@ -269,10 +263,10 @@ tds_read_config_info(TDSSOCKET * tds, TDSLOGIN * login, TDSLOCALE * locale)
 	/*
 	 * If a dump file has been specified, start logging
 	 */
-	if (!tds_dstr_isempty(&connection->dump_file) && !tdsdump_isopen()) {
+	if (connection->dump_file != NULL && !tdsdump_isopen()) {
 		if (connection->debug_flags)
 			tds_debug_flags = connection->debug_flags;
-		tdsdump_open(tds_dstr_cstr(&connection->dump_file));
+		tdsdump_topen(connection->dump_file);
 	}
 
 	return connection;
@@ -293,21 +287,21 @@ tds_fix_login(TDSLOGIN * login)
 }
 
 static bool
-tds_try_conf_file(const char *path, const char *how, const char *server, TDSLOGIN * login)
+tds_try_conf_file(const tds_dir_char *path, const char *how, const char *server, TDSLOGIN * login)
 {
 	bool found = false;
 	FILE *in;
 
-	if ((in = fopen(path, "r")) == NULL) {
-		tdsdump_log(TDS_DBG_INFO1, "Could not open '%s' (%s).\n", path, how);
+	if ((in = tds_dir_open(path, TDS_DIR("r"))) == NULL) {
+		tdsdump_log(TDS_DBG_INFO1, "Could not open '%" tdsPRIdir "' (%s).\n", path, how);
 		return found;
 	}
 
-	tdsdump_log(TDS_DBG_INFO1, "Found conf file '%s' %s.\n", path, how);
+	tdsdump_log(TDS_DBG_INFO1, "Found conf file '%" tdsPRIdir "' %s.\n", path, how);
 	found = tds_read_conf_sections(in, server, login);
 
 	if (found) {
-		tdsdump_log(TDS_DBG_INFO1, "Success: [%s] defined in %s.\n", server, path);
+		tdsdump_log(TDS_DBG_INFO1, "Success: [%s] defined in %" tdsPRIdir ".\n", server, path);
 	} else {
 		tdsdump_log(TDS_DBG_INFO2, "[%s] not found.\n", server);
 	}
@@ -317,25 +311,6 @@ tds_try_conf_file(const char *path, const char *how, const char *server, TDSLOGI
 	return found;
 }
 
-
-/**
- * Return filename from HOME directory
- * @return allocated string or NULL if error
- */
-char *
-tds_get_home_file(const char *file)
-{
-	char *home, *path;
-
-	home = tds_get_homedir();
-	if (!home)
-		return NULL;
-	if (asprintf(&path, "%s/%s", home, file) < 0)
-		path = NULL;
-	free(home);
-	return path;
-}
-
 /**
  * Read configuration info for given server
  * return 0 on error
@@ -346,8 +321,8 @@ tds_get_home_file(const char *file)
 bool
 tds_read_conf_file(TDSLOGIN * login, const char *server)
 {
-	char *path = NULL;
-	char *eptr = NULL;
+	tds_dir_char *path = NULL;
+	tds_dir_char *eptr = NULL;
 	bool found = false;
 
 	if (interf_file) {
@@ -356,7 +331,7 @@ tds_read_conf_file(TDSLOGIN * login, const char *server)
 
 	/* FREETDSCONF env var, pkleef@openlinksw.com 01/21/02 */
 	if (!found) {
-		path = getenv("FREETDSCONF");
+		path = tds_dir_getenv(TDS_DIR("FREETDSCONF"));
 		if (path) {
 			found = tds_try_conf_file(path, "(from $FREETDSCONF)", server, login);
 		} else {
@@ -366,9 +341,10 @@ tds_read_conf_file(TDSLOGIN * login, const char *server)
 
 	/* FREETDS env var, Bill Thompson 16/07/03 */
 	if (!found) {
-		eptr = getenv("FREETDS");
+		eptr = tds_dir_getenv(TDS_DIR("FREETDS"));
 		if (eptr) {
-			if (asprintf(&path, freetds_conf, eptr) >= 0) {
+			path = tds_join_path(eptr, freetds_conf);
+			if (path) {
 				found = tds_try_conf_file(path, location, server, login);
 				free(path);
 			}
@@ -377,13 +353,24 @@ tds_read_conf_file(TDSLOGIN * login, const char *server)
 		}
 	}
 
+#if !defined(_WIN32) && !defined(DOS32X)
 	if (!found) {
-		path = tds_get_home_file(".freetds.conf");
+		path = tds_get_home_file(TDS_DIR(".config/freetds.conf"));
+		if (path) {
+			found = tds_try_conf_file(path, "(.config/freetds.conf)", server, login);
+			free(path);
+		}
+	}
+#endif
+
+	if (!found) {
+		path = tds_get_home_file(TDS_DIR(".freetds.conf"));
 		if (path) {
 			found = tds_try_conf_file(path, "(.freetds.conf)", server, login);
 			free(path);
 		} else {
-			tdsdump_log(TDS_DBG_INFO2, "... Error getting ~/.freetds.conf.  Trying %s.\n", FREETDS_SYSCONFFILE);
+			tdsdump_log(TDS_DBG_INFO2, "... Error getting ~/.freetds.conf.  Trying %" tdsPRIdir ".\n",
+				    FREETDS_SYSCONFFILE);
 		}
 	}
 
@@ -470,7 +457,20 @@ tds_config_boolean(const char *option, const char *value, TDSLOGIN *login)
 	return 0;
 }
 
-static void
+static int
+tds_parse_boolean_option(const char *option, const char *value, int default_value, bool *p_error)
+{
+	int ret = tds_parse_boolean(value, -1);
+	if (ret >= 0)
+		return ret;
+
+	tdsdump_log(TDS_DBG_ERROR, "UNRECOGNIZED option value '%s' for boolean setting '%s'!\n",
+		    value, option);
+	*p_error = true;
+	return default_value;
+}
+
+static bool
 tds_config_encryption(const char * value, TDSLOGIN * login)
 {
 	TDS_ENCRYPTION_LEVEL lvl = TDS_ENCRYPTION_OFF;
@@ -481,16 +481,20 @@ tds_config_encryption(const char * value, TDSLOGIN * login)
 		lvl = TDS_ENCRYPTION_REQUEST;
 	else if (!strcasecmp(value, TDS_STR_ENCRYPTION_REQUIRE))
 		lvl = TDS_ENCRYPTION_REQUIRE;
+	else if (!strcasecmp(value, TDS_STR_ENCRYPTION_STRICT))
+		lvl = TDS_ENCRYPTION_STRICT;
 	else {
 		tdsdump_log(TDS_DBG_ERROR, "UNRECOGNIZED option value '%s' for '%s' setting!\n",
 			    value, TDS_STR_ENCRYPTION);
-		tdsdump_log(TDS_DBG_ERROR, "Valid settings are: ('%s', '%s', '%s')\n",
-		        TDS_STR_ENCRYPTION_OFF, TDS_STR_ENCRYPTION_REQUEST, TDS_STR_ENCRYPTION_REQUIRE);
+		tdsdump_log(TDS_DBG_ERROR, "Valid settings are: ('%s', '%s', '%s', '%s')\n",
+		        TDS_STR_ENCRYPTION_OFF, TDS_STR_ENCRYPTION_REQUEST, TDS_STR_ENCRYPTION_REQUIRE,
+			TDS_STR_ENCRYPTION_STRICT);
 		lvl = TDS_ENCRYPTION_REQUIRE;  /* Assuming "require" is safer than "no" */
-		login->valid_configuration = 0;
+		return false;
 	}
 
 	login->encryption_level = lvl;
+	return true;
 }
 
 /**
@@ -508,7 +512,7 @@ tds_read_conf_section(FILE * in, const char *section, TDSCONFPARSE tds_conf_pars
 	char *s;
 	char p;
 	int i;
-	int insection = 0;
+	bool insection = false;
 	bool found = false;
 
 	tdsdump_log(TDS_DBG_INFO1, "Looking for section %s.\n", section);
@@ -574,10 +578,10 @@ tds_read_conf_section(FILE * in, const char *section, TDSCONFPARSE tds_conf_pars
 
 			if (!strcasecmp(section, &option[1])) {
 				tdsdump_log(TDS_DBG_INFO1, "Got a match.\n");
-				insection = 1;
+				insection = true;
 				found = true;
 			} else {
-				insection = 0;
+				insection = false;
 			}
 		} else if (insection) {
 			tds_conf_parse(option, value, param);
@@ -590,11 +594,15 @@ tds_read_conf_section(FILE * in, const char *section, TDSCONFPARSE tds_conf_pars
 }
 
 /* Also used to scan ODBC.INI entries */
-void
+bool
 tds_parse_conf_section(const char *option, const char *value, void *param)
 {
+#define parse_boolean(option, value, variable) do { \
+	variable = tds_parse_boolean_option(option, value, variable, &got_error); \
+} while(0)
 	TDSLOGIN *login = (TDSLOGIN *) param;
 	void *s = param;
+	bool got_error = false;
 
 	tdsdump_log(TDS_DBG_INFO1, "\t%s = '%s'\n", option, value);
 
@@ -609,9 +617,16 @@ tds_parse_conf_section(const char *option, const char *value, void *param)
 		tds_config_boolean(option, value, login);
 	} else if (!strcmp(option, TDS_GSSAPI_DELEGATION)) {
 		/* gssapi flag addition */
-		login->gssapi_use_delegation = tds_config_boolean(option, value, login);
+		parse_boolean(option, value, login->gssapi_use_delegation);
+	} else if (!strcmp(option, TDS_STR_MUTUAL_AUTHENTICATION)) {
+		parse_boolean(option, value, login->mutual_authentication);
 	} else if (!strcmp(option, TDS_STR_DUMPFILE)) {
-		s = tds_dstr_copy(&login->dump_file, value);
+		TDS_ZERO_FREE(login->dump_file);
+		if (value[0]) {
+			login->dump_file = tds_dir_from_cstr(value);
+			if (!login->dump_file)
+				s = NULL;
+		}
 	} else if (!strcmp(option, TDS_STR_DEBUGFLAGS)) {
 		char *end;
 		long flags;
@@ -630,7 +645,7 @@ tds_parse_conf_section(const char *option, const char *value, void *param)
 
 		if (TDS_FAILED(tds_lookup_host_set(value, &login->ip_addrs))) {
 			tdsdump_log(TDS_DBG_WARN, "Found host entry %s however name resolution failed. \n", value);
-			return;
+			return false;
 		}
 
 		tdsdump_log(TDS_DBG_INFO1, "Found host entry %s \n", value);
@@ -642,7 +657,8 @@ tds_parse_conf_section(const char *option, const char *value, void *param)
 		if (atoi(value))
 			login->port = atoi(value);
 	} else if (!strcmp(option, TDS_STR_EMUL_LE)) {
-		login->emul_little_endian = tds_config_boolean(option, value, login);
+		/* obsolete, ignore */
+		tds_config_boolean(option, value, login);
 	} else if (!strcmp(option, TDS_STR_TEXTSZ)) {
 		if (atoi(value))
 			login->text_size = atoi(value);
@@ -653,22 +669,23 @@ tds_parse_conf_section(const char *option, const char *value, void *param)
 		s = tds_dstr_copy(&login->client_charset, value);
 		tdsdump_log(TDS_DBG_INFO1, "tds_parse_conf_section: %s is %s.\n", option, tds_dstr_cstr(&login->client_charset));
 	} else if (!strcmp(option, TDS_STR_USE_UTF_16)) {
-		login->use_utf16 = tds_config_boolean(option, value, login);
+		parse_boolean(option, value, login->use_utf16);
 	} else if (!strcmp(option, TDS_STR_LANGUAGE)) {
 		s = tds_dstr_copy(&login->language, value);
 	} else if (!strcmp(option, TDS_STR_APPENDMODE)) {
-		tds_g_append_mode = tds_config_boolean(option, value, login);
+		parse_boolean(option, value, tds_append_mode);
 	} else if (!strcmp(option, TDS_STR_INSTANCE)) {
 		s = tds_dstr_copy(&login->instance_name, value);
 	} else if (!strcmp(option, TDS_STR_ENCRYPTION)) {
-		tds_config_encryption(value, login);
+		if (!tds_config_encryption(value, login))
+			s = NULL;
 	} else if (!strcmp(option, TDS_STR_ASA_DATABASE)) {
 		s = tds_dstr_copy(&login->server_name, value);
 	} else if (!strcmp(option, TDS_STR_USENTLMV2)) {
-		login->use_ntlmv2 = tds_config_boolean(option, value, login);
+		parse_boolean(option, value, login->use_ntlmv2);
 		login->use_ntlmv2_specified = 1;
 	} else if (!strcmp(option, TDS_STR_USELANMAN)) {
-		login->use_lanman = tds_config_boolean(option, value, login);
+		parse_boolean(option, value, login->use_lanman);
 	} else if (!strcmp(option, TDS_STR_REALM)) {
 		s = tds_dstr_copy(&login->server_realm_name, value);
 	} else if (!strcmp(option, TDS_STR_SPN)) {
@@ -678,86 +695,106 @@ tds_parse_conf_section(const char *option, const char *value, void *param)
 	} else if (!strcmp(option, TDS_STR_CRLFILE)) {
 		s = tds_dstr_copy(&login->crlfile, value);
 	} else if (!strcmp(option, TDS_STR_CHECKSSLHOSTNAME)) {
-		login->check_ssl_hostname = tds_config_boolean(option, value, login);
+		parse_boolean(option, value, login->check_ssl_hostname);
+	} else if (!strcmp(option, TDS_STR_SSLHOSTNAME)) {
+		s = tds_dstr_copy(&login->certificate_host_name, value);
 	} else if (!strcmp(option, TDS_STR_DBFILENAME)) {
 		s = tds_dstr_copy(&login->db_filename, value);
 	} else if (!strcmp(option, TDS_STR_DATABASE)) {
 		s = tds_dstr_copy(&login->database, value);
 	} else if (!strcmp(option, TDS_STR_READONLY_INTENT)) {
-		login->readonly_intent = tds_config_boolean(option, value, login);
+		parse_boolean(option, value, login->readonly_intent);
 		tdsdump_log(TDS_DBG_FUNC, "Setting ReadOnly Intent to '%s'.\n", value);
 	} else if (!strcmp(option, TLS_STR_OPENSSL_CIPHERS)) {
 		s = tds_dstr_copy(&login->openssl_ciphers, value);
 	} else if (!strcmp(option, TDS_STR_ENABLE_TLS_V1)) {
-		login->enable_tls_v1 = tds_config_boolean(option, value, login);
+		parse_boolean(option, value, login->enable_tls_v1);
+		login->enable_tls_v1_specified = 1;
 	} else {
 		tdsdump_log(TDS_DBG_INFO1, "UNRECOGNIZED option '%s' ... ignoring.\n", option);
 	}
 
-	if (!s)
+	if (!s || got_error) {
 		login->valid_configuration = 0;
+		return false;
+	}
+	return true;
+#undef parse_boolean
 }
 
-static int
+static bool
 tds_config_login(TDSLOGIN * connection, TDSLOGIN * login)
 {
 	DSTR *res = &login->server_name;
 
 	if (!tds_dstr_isempty(&login->server_name)) {
-		if (1 || tds_dstr_isempty(&connection->server_name)) 
+		if (1 || tds_dstr_isempty(&connection->server_name))
 			res = tds_dstr_dup(&connection->server_name, &login->server_name);
 	}
+
 	if (login->tds_version)
 		connection->tds_version = login->tds_version;
-	if (res && !tds_dstr_isempty(&login->language)) {
+
+	if (res && !tds_dstr_isempty(&login->language))
 		res = tds_dstr_dup(&connection->language, &login->language);
-	}
-	if (res && !tds_dstr_isempty(&login->server_charset)) {
+
+	if (res && !tds_dstr_isempty(&login->server_charset))
 		res = tds_dstr_dup(&connection->server_charset, &login->server_charset);
-	}
+
 	if (res && !tds_dstr_isempty(&login->client_charset)) {
 		res = tds_dstr_dup(&connection->client_charset, &login->client_charset);
 		tdsdump_log(TDS_DBG_INFO1, "tds_config_login: %s is %s.\n", "client_charset",
 			    tds_dstr_cstr(&connection->client_charset));
 	}
+
 	if (!login->use_utf16)
 		connection->use_utf16 = login->use_utf16;
+
 	if (res && !tds_dstr_isempty(&login->database)) {
 		res = tds_dstr_dup(&connection->database, &login->database);
 		tdsdump_log(TDS_DBG_INFO1, "tds_config_login: %s is %s.\n", "database_name",
 			    tds_dstr_cstr(&connection->database));
 	}
-	if (res && !tds_dstr_isempty(&login->client_host_name)) {
+
+	if (res && !tds_dstr_isempty(&login->client_host_name))
 		res = tds_dstr_dup(&connection->client_host_name, &login->client_host_name);
-	}
-	if (res && !tds_dstr_isempty(&login->app_name)) {
+
+	if (res && !tds_dstr_isempty(&login->app_name))
 		res = tds_dstr_dup(&connection->app_name, &login->app_name);
-	}
-	if (res && !tds_dstr_isempty(&login->user_name)) {
+
+	if (res && !tds_dstr_isempty(&login->user_name))
 		res = tds_dstr_dup(&connection->user_name, &login->user_name);
-	}
+
 	if (res && !tds_dstr_isempty(&login->password)) {
 		/* for security reason clear memory */
 		tds_dstr_zero(&connection->password);
 		res = tds_dstr_dup(&connection->password, &login->password);
 	}
-	if (res && !tds_dstr_isempty(&login->library)) {
+
+	if (res && !tds_dstr_isempty(&login->library))
 		res = tds_dstr_dup(&connection->library, &login->library);
-	}
-	if (login->encryption_level) {
+
+	if (login->encryption_level)
 		connection->encryption_level = login->encryption_level;
-	}
-	if (login->suppress_language) {
+
+	if (login->suppress_language)
 		connection->suppress_language = 1;
-	}
-	if (!login->bulk_copy) {
+
+	if (!login->bulk_copy)
 		connection->bulk_copy = 0;
-	}
-	if (login->block_size) {
+
+	if (login->block_size)
 		connection->block_size = login->block_size;
-	}
+
+	if (login->gssapi_use_delegation)
+		connection->gssapi_use_delegation = login->gssapi_use_delegation;
+
+	if (login->mutual_authentication)
+		connection->mutual_authentication = login->mutual_authentication;
+
 	if (login->port)
 		connection->port = login->port;
+
 	if (login->connect_timeout)
 		connection->connect_timeout = login->connect_timeout;
 
@@ -767,19 +804,21 @@ tds_config_login(TDSLOGIN * connection, TDSLOGIN * login)
 	if (!login->check_ssl_hostname)
 		connection->check_ssl_hostname = login->check_ssl_hostname;
 
-	if (res && !tds_dstr_isempty(&login->db_filename)) {
+	if (res && !tds_dstr_isempty(&login->db_filename))
 		res = tds_dstr_dup(&connection->db_filename, &login->db_filename);
-	}
 
-	if (res && !tds_dstr_isempty(&login->openssl_ciphers)) {
+	if (res && !tds_dstr_isempty(&login->openssl_ciphers))
 		res = tds_dstr_dup(&connection->openssl_ciphers, &login->openssl_ciphers);
-	}
+
+	if (res && !tds_dstr_isempty(&login->server_spn))
+		res = tds_dstr_dup(&connection->server_spn, &login->server_spn);
 
 	/* copy other info not present in configuration file */
 	connection->capabilities = login->capabilities;
 
 	if (login->readonly_intent)
 		connection->readonly_intent = login->readonly_intent;
+
 	connection->use_new_password = login->use_new_password;
 
 	if (login->use_ntlmv2_specified) {
@@ -787,34 +826,39 @@ tds_config_login(TDSLOGIN * connection, TDSLOGIN * login)
 		connection->use_ntlmv2 = login->use_ntlmv2;
 	}
 
+	if (login->enable_tls_v1_specified) {
+		connection->enable_tls_v1_specified = login->enable_tls_v1_specified;
+		connection->enable_tls_v1 = login->enable_tls_v1;
+	}
+
 	if (res)
 		res = tds_dstr_dup(&connection->new_password, &login->new_password);
 
 	return res != NULL;
 }
 
-static int
+static bool
 tds_config_env_tdsdump(TDSLOGIN * login)
 {
-	char *s = getenv("TDSDUMP");
-	if (!s)
-		return 1;
+	tds_dir_char path[TDS_VECTOR_SIZE(pid_logpath) + 22];
 
-	if (!strlen(s)) {
-		char *path;
+	tds_dir_char *s = tds_dir_getenv(TDS_DIR("TDSDUMP"));
+	if (!s)
+		return true;
+
+	if (!tds_dir_len(s)) {
 		pid_t pid = getpid();
-		if (asprintf(&path, pid_logpath, (int) pid) < 0)
-			return 0;
-		if (!tds_dstr_set(&login->dump_file, path)) {
-			free(path);
-			return 0;
-		}
-	} else {
-		if (!tds_dstr_copy(&login->dump_file, s))
-			return 0;
+		tds_dir_snprintf(path, TDS_VECTOR_SIZE(path), pid_logpath, (int) pid);
+		s = path;
 	}
-	tdsdump_log(TDS_DBG_INFO1, "Setting 'dump_file' to '%s' from $TDSDUMP.\n", tds_dstr_cstr(&login->dump_file));
-	return 1;
+
+	if (!(s = tds_dir_dup(s)))
+		return false;
+	free(login->dump_file);
+	login->dump_file = s;
+
+	tdsdump_log(TDS_DBG_INFO1, "Setting 'dump_file' to '%" tdsPRIdir "' from $TDSDUMP.\n", login->dump_file);
+	return true;
 }
 
 static void
@@ -843,7 +887,7 @@ tds_config_env_tdsver(TDSLOGIN * login)
 }
 
 /* TDSHOST env var, pkleef@openlinksw.com 01/21/02 */
-static int
+static bool
 tds_config_env_tdshost(TDSLOGIN * login)
 {
 	const char *tdshost;
@@ -851,34 +895,35 @@ tds_config_env_tdshost(TDSLOGIN * login)
 	struct addrinfo *addrs;
 
 	if (!(tdshost = getenv("TDSHOST")))
-		return 1;
+		return true;
 
 	if (TDS_FAILED(tds_lookup_host_set(tdshost, &login->ip_addrs))) {
 		tdsdump_log(TDS_DBG_WARN, "Name resolution failed for '%s' from $TDSHOST.\n", tdshost);
-		return 0;
+		return false;
 	}
 
 	if (!tds_dstr_copy(&login->server_host_name, tdshost))
-		return 0;
+		return false;
 	for (addrs = login->ip_addrs; addrs != NULL; addrs = addrs->ai_next) {
 		tdsdump_log(TDS_DBG_INFO1, "Setting IP Address to %s (%s) from $TDSHOST.\n",
 			    tds_addrinfo2str(addrs, tmp, sizeof(tmp)), tdshost);
 	}
-	return 1;
+	return true;
 }
-#define TDS_FIND(k,b,c) tds_find(k, b, sizeof(b)/sizeof(b[0]), sizeof(b[0]), c)
+#define TDS_FIND(k,b,c) tds_find(k, b, TDS_VECTOR_SIZE(b), sizeof(b[0]), c)
 
 
-static void *
+static const void *
 tds_find(const void *key, const void *base, size_t nelem, size_t width,
          int (*compar)(const void *, const void *))
 {
-	size_t i;
-	for (i=0; i < nelem; i++) {
-		char *p = (char*)base + width * i;
-		if (0 == compar(key, p)) {
+	size_t n;
+	const char *p = (const char*) base;
+
+	for (n = nelem; n != 0; --n) {
+		if (0 == compar(key, p))
 			return p;
-		}
+		p += width;
 	}
 	return NULL;
 }
@@ -890,16 +935,16 @@ struct tdsvername_t
 };
 
 static int
-tds_vernanme_cmp(const void *key, const void *pelem)
+tds_vername_cmp(const void *key, const void *pelem)
 {
 	return strcmp((const char *)key, ((const struct tdsvername_t *)pelem)->name);
-}	
+}
 
 /**
  * Set TDS version from given string
  * @param tdsver tds string version
  * @param login where to store information
- * @return as encoded hex value: high nybble major, low nybble minor.
+ * @return as encoded hex value: high byte major, low byte minor.
  */
 TDS_USMALLINT *
 tds_config_verstr(const char *tdsver, TDSLOGIN * login)
@@ -908,19 +953,15 @@ tds_config_verstr(const char *tdsver, TDSLOGIN * login)
 		{ {   "0", 0x000 }
 		, {"auto", 0x000 }
 		, { "4.2", 0x402 }
-		, { "4.2", 0x402 }
-		, {  "46", 0x406 }
-		, { "4.6", 0x406 }
 		, {  "50", 0x500 }
 		, { "5.0", 0x500 }
 		, {  "70", 0x700 }
 		, { "7.0", 0x700 }
-		, {  "80", 0x701 }
-		, { "8.0", 0x701 }
 		, { "7.1", 0x701 }
 		, { "7.2", 0x702 }
 		, { "7.3", 0x703 }
 		, { "7.4", 0x704 }
+		, { "8.0", 0x800 }
 		};
 	const struct tdsvername_t *pver;
 
@@ -929,7 +970,7 @@ tds_config_verstr(const char *tdsver, TDSLOGIN * login)
 		return NULL;
 	}
 
-	if ((pver = (const struct tdsvername_t *) TDS_FIND(tdsver, tds_versions, tds_vernanme_cmp)) == NULL) {
+	if ((pver = (const struct tdsvername_t *) TDS_FIND(tdsver, tds_versions, tds_vername_cmp)) == NULL) {
 		tdsdump_log(TDS_DBG_INFO1, "error: no such version: %s\n", tdsver);
 		return NULL;
 	}
@@ -955,7 +996,7 @@ tds_set_interfaces_file_loc(const char *interf)
 		return TDS_SUCCESS;
 	}
 	/* Set to new value */
-	if ((interf_file = strdup(interf)) == NULL) {
+	if ((interf_file = tds_dir_from_cstr(interf)) == NULL) {
 		return TDS_FAIL;
 	}
 	return TDS_SUCCESS;
@@ -979,9 +1020,22 @@ tds_lookup_host(const char *servername)	/* (I) name of the server
 	memset(&hints, '\0', sizeof(hints));
 	hints.ai_family = AF_UNSPEC;
 	hints.ai_socktype = SOCK_STREAM;
+	hints.ai_protocol = IPPROTO_TCP;
 
 #ifdef AI_ADDRCONFIG
 	hints.ai_flags |= AI_ADDRCONFIG;
+	switch (getaddrinfo(servername, NULL, &hints, &addr)) {
+	case 0:
+		return addr;
+	case EAI_FAMILY:
+#  ifdef EAI_ADDRFAMILY
+	case EAI_ADDRFAMILY:
+#  endif
+		hints.ai_flags &= ~AI_ADDRCONFIG;
+		break;
+	default:
+		return NULL;
+	}
 #endif
 
 	if (getaddrinfo(servername, NULL, &hints, &addr))
@@ -1024,7 +1078,7 @@ hexdigit(int c)
 {
 	if (c >= '0' && c <= '9')
 		return c - '0';
-	/* ascii optimization, 'A' -> 'a', 'a' -> 'a' */
+	/* ASCII optimization, 'A' -> 'a', 'a' -> 'a' */
 	c |= 0x20;
 	if (c >= 'a' && c <= 'f')
 		return c - 'a' + 10;
@@ -1045,20 +1099,20 @@ hex2num(char *hex)
  * \param dir name of base directory for interface file
  * \param file name of the interface file
  * \param host logical host to search for
- * \return 0 if not fount 1 if found
+ * \return false if not fount true if found
  */
-static int
-search_interface_file(TDSLOGIN * login, const char *dir, const char *file, const char *host)
+static bool
+search_interface_file(TDSLOGIN * login, const tds_dir_char *dir, const tds_dir_char *file, const char *host)
 {
-	char *pathname;
+	tds_dir_char *pathname;
 	char line[255];
 	char tmp_ip[sizeof(line)];
 	char tmp_port[sizeof(line)];
 	char tmp_ver[sizeof(line)];
 	FILE *in;
 	char *field;
-	int found = 0;
-	int server_found = 0;
+	bool found = false;
+	bool server_found = false;
 	char *lasts;
 
 	line[0] = '\0';
@@ -1066,36 +1120,28 @@ search_interface_file(TDSLOGIN * login, const char *dir, const char *file, const
 	tmp_port[0] = '\0';
 	tmp_ver[0] = '\0';
 
-	tdsdump_log(TDS_DBG_INFO1, "Searching interfaces file %s/%s.\n", dir, file);
-	pathname = tds_new(char, strlen(dir) + strlen(file) + 10);
-	if (!pathname)
-		return 0;
+	tdsdump_log(TDS_DBG_INFO1, "Searching interfaces file %" tdsPRIdir "/%" tdsPRIdir ".\n", dir, file);
 
 	/*
 	 * create the full pathname to the interface file
 	 */
 	if (file[0] == '\0') {
-		pathname[0] = '\0';
+		pathname = tds_dir_dup(TDS_DIR(""));
 	} else {
-		if (dir[0] == '\0') {
-			pathname[0] = '\0';
-		} else {
-			strcpy(pathname, dir);
-			strcat(pathname, TDS_SDIR_SEPARATOR);
-		}
-		strcat(pathname, file);
+		pathname = tds_join_path(dir, file);
 	}
-
+	if (!pathname)
+		return false;
 
 	/*
 	 * parse the interfaces file and find the server and port
 	 */
-	if ((in = fopen(pathname, "r")) == NULL) {
-		tdsdump_log(TDS_DBG_INFO1, "Couldn't open %s.\n", pathname);
+	if ((in = tds_dir_open(pathname, TDS_DIR("r"))) == NULL) {
+		tdsdump_log(TDS_DBG_INFO1, "Couldn't open %" tdsPRIdir ".\n", pathname);
 		free(pathname);
-		return 0;
+		return false;
 	}
-	tdsdump_log(TDS_DBG_INFO1, "Interfaces file %s opened.\n", pathname);
+	tdsdump_log(TDS_DBG_INFO1, "Interfaces file %" tdsPRIdir " opened.\n", pathname);
 
 	while (fgets(line, sizeof(line) - 1, in)) {
 		if (line[0] == '#')
@@ -1104,10 +1150,10 @@ search_interface_file(TDSLOGIN * login, const char *dir, const char *file, const
 		if (!TDS_ISSPACE(line[0])) {
 			field = strtok_r(line, "\n\t ", &lasts);
 			if (!strcmp(field, host)) {
-				found = 1;
+				found = true;
 				tdsdump_log(TDS_DBG_INFO1, "Found matching entry for host %s.\n", host);
 			} else
-				found = 0;
+				found = false;
 		} else if (found && TDS_ISSPACE(line[0])) {
 			field = strtok_r(line, "\n\t ", &lasts);
 			if (field != NULL && !strcmp(field, "query")) {
@@ -1132,7 +1178,7 @@ search_interface_file(TDSLOGIN * login, const char *dir, const char *file, const
 					field = strtok_r(NULL, "\n\t ", &lasts);	/* port */
 					strcpy(tmp_port, field);
 				}	/* else */
-				server_found = 1;
+				server_found = true;
 			}	/* if */
 		}		/* else if */
 	}			/* while */
@@ -1148,7 +1194,7 @@ search_interface_file(TDSLOGIN * login, const char *dir, const char *file, const
 		if (TDS_SUCCEED(tds_lookup_host_set(tmp_ip, &login->ip_addrs))) {
 			struct addrinfo *addrs;
 			if (!tds_dstr_copy(&login->server_host_name, tmp_ip))
-				return 0;
+				return false;
 			for (addrs = login->ip_addrs; addrs != NULL; addrs = addrs->ai_next) {
 				tdsdump_log(TDS_DBG_INFO1, "Resolved IP as '%s'.\n",
 					    tds_addrinfo2str(login->ip_addrs, line, sizeof(line)));
@@ -1170,10 +1216,10 @@ search_interface_file(TDSLOGIN * login, const char *dir, const char *file, const
  *
  * @note This function uses only the interfaces file and is deprecated.
  */
-static int
+static bool
 tds_read_interfaces(const char *server, TDSLOGIN * login)
 {
-	int found = 0;
+	bool found = false;
 
 	/* read $SYBASE/interfaces */
 
@@ -1190,19 +1236,19 @@ tds_read_interfaces(const char *server, TDSLOGIN * login)
 	 * Look for the server in the interf_file iff interf_file has been set.
 	 */
 	if (interf_file) {
-		tdsdump_log(TDS_DBG_INFO1, "Looking for server in file %s.\n", interf_file);
-		found = search_interface_file(login, "", interf_file, server);
+		tdsdump_log(TDS_DBG_INFO1, "Looking for server in file %" tdsPRIdir ".\n", interf_file);
+		found = search_interface_file(login, TDS_DIR(""), interf_file, server);
 	}
 
 	/*
 	 * if we haven't found the server yet then look for a $HOME/.interfaces file
 	 */
 	if (!found) {
-		char *path = tds_get_home_file(".interfaces");
+		tds_dir_char *path = tds_get_home_file(TDS_DIR(".interfaces"));
 
 		if (path) {
-			tdsdump_log(TDS_DBG_INFO1, "Looking for server in %s.\n", path);
-			found = search_interface_file(login, "", path, server);
+			tdsdump_log(TDS_DBG_INFO1, "Looking for server in %" tdsPRIdir ".\n", path);
+			found = search_interface_file(login, TDS_DIR(""), path, server);
 			free(path);
 		}
 	}
@@ -1211,7 +1257,7 @@ tds_read_interfaces(const char *server, TDSLOGIN * login)
 	 * if we haven't found the server yet then look in $SYBBASE/interfaces file
 	 */
 	if (!found) {
-		const char *sybase = getenv("SYBASE");
+		const tds_dir_char *sybase = tds_dir_getenv(TDS_DIR("SYBASE"));
 #ifdef __VMS
 		/* We've got to be in unix syntax for later slash-joined concatenation. */
 		#include <unixlib.h>
@@ -1221,8 +1267,8 @@ tds_read_interfaces(const char *server, TDSLOGIN * login)
 		if (!sybase || !sybase[0])
 			sybase = interfaces_path;
 
-		tdsdump_log(TDS_DBG_INFO1, "Looking for server in %s/interfaces.\n", sybase);
-		found = search_interface_file(login, sybase, "interfaces", server);
+		tdsdump_log(TDS_DBG_INFO1, "Looking for server in %" tdsPRIdir "/interfaces.\n", sybase);
+		found = search_interface_file(login, sybase, TDS_DIR("interfaces"), server);
 	}
 
 	/*
@@ -1262,7 +1308,7 @@ tds_read_interfaces(const char *server, TDSLOGIN * login)
 
 		if (TDS_SUCCEED(tds_lookup_host_set(server, &login->ip_addrs)))
 			if (!tds_dstr_copy(&login->server_host_name, server))
-				return 0;
+				return false;
 
 		if (ip_port)
 			login->port = ip_port;
@@ -1274,9 +1320,9 @@ tds_read_interfaces(const char *server, TDSLOGIN * login)
 /**
  * Check the server name to find port info first
  * Warning: connection-> & login-> are all modified when needed
- * \return 1 when found, else 0
+ * \return true when found, else false
  */
-static int
+static bool
 parse_server_name_for_port(TDSLOGIN * connection, TDSLOGIN * login, bool update_server)
 {
 	const char *pSep;
@@ -1302,17 +1348,24 @@ parse_server_name_for_port(TDSLOGIN * connection, TDSLOGIN * login, bool update_
 		/* handle instance name */
 		pSep = strrchr(server, '\\');
 		if (!pSep || pSep == server)
-			return 0;
+			return false;
 
 		if (!tds_dstr_copy(&connection->instance_name, pSep + 1))
-			return 0;
+			return false;
 		connection->port = 0;
 	}
 
-	if (!update_server || !tds_dstr_copyn(&connection->server_name, server, pSep - server))
-		return 0;
+	if (!update_server)
+		return true;
 
-	return 1;
+	if (server[0] == '[' && pSep > server && pSep[-1] == ']') {
+		server++;
+		pSep--;
+	}
+	if (!tds_dstr_copyn(&connection->server_name, server, pSep - server))
+		return false;
+
+	return true;
 }
 
 /**
@@ -1327,26 +1380,6 @@ tds_get_compiletime_settings(void)
 		  TDS_VERSION_NO
 		, FREETDS_SYSCONFDIR
 		, "unknown"	/* need fancy script in makefile */
-#		ifdef MSDBLIB
-			, 1
-#		else
-			, 0
-#		endif
-#		ifdef TDS_SYBASE_COMPAT
-			, 1
-#		else
-			, 0
-#		endif
-#		ifdef _REENTRANT
-			, 1
-#		else
-			, 0
-#		endif
-#		ifdef HAVE_ICONV
-			, 1
-#		else
-			, 0
-#		endif
 #		if TDS50
 			, "5.0"
 #		elif TDS71
@@ -1360,30 +1393,60 @@ tds_get_compiletime_settings(void)
 #		else
 			, "auto"
 #		endif
-#		ifdef IODBC
-			, 1
+#		ifdef MSDBLIB
+			, true
 #		else
-			, 0
+			, false
+#		endif
+#		ifdef TDS_SYBASE_COMPAT
+			, true
+#		else
+			, false
+#		endif
+#		ifdef _REENTRANT
+			, true
+#		else
+			, false
+#		endif
+#		ifdef HAVE_ICONV
+			, true
+#		else
+			, false
+#		endif
+#		ifdef IODBC
+			, true
+#		else
+			, false
 #		endif
 #		ifdef UNIXODBC
-			, 1
+			, true
 #		else
-			, 0
+			, false
 #		endif
 #		ifdef HAVE_OPENSSL
-			, 1
+			, true
 #		else
-			, 0
+			, false
 #		endif
 #		ifdef HAVE_GNUTLS
-			, 1
+			, true
 #		else
-			, 0
+			, false
 #		endif
 #		if ENABLE_ODBC_MARS
-			, 1
+			, true
 #		else
-			, 0
+			, false
+#		endif
+#		ifdef HAVE_SSPI
+			, true
+#		else
+			, false
+#		endif
+#		ifdef ENABLE_KRB5
+			, true
+#		else
+			, false
 #		endif
 	};
 
@@ -1392,4 +1455,25 @@ tds_get_compiletime_settings(void)
 	return &settings;
 }
 
+/**
+ * Make sure proper setting are in place for TDS 8.0
+ */
+TDSRET
+tds8_adjust_login(TDSLOGIN *login)
+{
+	if (!IS_TDS80_PLUS(login) && login->encryption_level != TDS_ENCRYPTION_STRICT)
+		return TDS_SUCCESS;
+
+	login->tds_version = 0x800;
+	login->encryption_level = TDS_ENCRYPTION_STRICT;
+
+	/* we must have certificates */
+	if (tds_dstr_isempty(&login->cafile)) {
+		if (!tds_dstr_copy(&login->cafile, "system"))
+			return -TDSEMEM;
+	}
+
+	return TDS_SUCCESS;
+}
+
 /** @} */
